
/* --------------------------------------------------------------------
 Evaluation of kernel functions: linear, poly, rbf, sigmoidal.

 About: Statistical Pattern Recognition Toolbox
 (C) 1999-2007, Written by Vojtech Franc and Vaclav Hlavac
 <a href="http://www.cvut.cz">Czech Technical University Prague</a>
 <a href="http://www.feld.cvut.cz">Faculty of Electrical Engineering</a>
 <a href="http://cmp.felk.cvut.cz">Center for Machine Perception</a>

 Modifications:
 05-mar-07, VF, added example on defining a customized kernel. 
 04-may-04, VF
 04-jun-03, VF
 12-nov-01, VF, sigmoid kenel
 22-sep-01, V. Franc, created.
-------------------------------------------------------------------- */

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <string.h>

/* --- Global variables --------------------------------------------- */

double *dataA;      /* pointer at the fist patterns */
double *dataB;      /* pointer at the second patterns */
long dim;           /* dimension of patterns */
int ker;            /* kernel id (0 - linear, 1 - polynomial, 
                       2 - rbf, 3 - sigmoid */
double *arg1;       /* kernel argument */
int	   arg1M;       /* kernel argument Rows*/
int	   arg1N;       /* kernel argument Columns*/

double *arg2;       /* kernel argument */
int	   arg2M;       /* kernel argument Rows*/
int	   arg2N;       /* kernel argument Columns*/

long ker_cnt;       /* number of cernel evaluations */

char *kernel_name[] = {"linear","poly","rbf","sigmoid","custom","bochner","CosineMixture"};

/* -------------------------------------------------------------------
 Computes dot product of a-th and b-th vector.
 c = a'*b
------------------------------------------------------------------- */
double dot_prod( long a, long b)
{
   double c = 0;
   long i;
   for( i = 0; i < dim; i++ ) {
      c += *(dataA+(a*dim)+i) * *(dataB+(b*dim)+i);
   }
   return( c );
}

double bochner( long a, long b)
{
   double c = 0;
   double gamma[2] = { -3.3375, 1.3449};
   long i;
   for( i = 0; i < dim; i++ ) {
      c += (*(dataA+(a*dim)+i) - *(dataB+(b*dim)+i)) * gamma[i];
   }
   return( cos(c) );
}


double CosineMixture( long a, long b)
{
   double c = 0, r = 0;
   long i, j;

   for (j=0; j<arg1N; j++)
   {
	   c = 0;
	    	/* dim = arg1M */
   		for( i = 0; i < dim; i++ ) {
    		c += (*(dataA+(a*dim)+i) - *(dataB+(b*dim)+i)) * arg1[j * arg1M + i];
		}
       r += arg2[j] * cos(c);
   }

   return( r );
}


/* -------------------------------------------------------------------
 Computes dot product of subtraction of a-th and b-th vector.
 c = (a-b)'*(a-b)
------------------------------------------------------------------- */
double sub_dot_prod( long a, long b )
{
   double c = 0;
   long i;
   for( i = 0; i < dim; i++ ) {
      c += (*(dataA+(a*dim)+i) - *(dataB+(b*dim)+i))*
           (*(dataA+(a*dim)+i) - *(dataB+(b*dim)+i));
   }
   return( c );
}

/* --------------------------------------------------------------------
 Converts string kernel identifier to int.
-------------------------------------------------------------------- */
int kernel_id( const mxArray *prhs1 )
{
  int num, i, buf_len;
  char *buf;

  if( mxIsChar( prhs1 ) != 1) return( -1 );

  buf_len  = (mxGetM(prhs1) * mxGetN(prhs1)) + 1;
  buf = mxCalloc( buf_len, sizeof( char ));

  mxGetString( prhs1, buf, buf_len );
  
  num = sizeof( kernel_name )/sizeof( char * );

  for( i = 0; i < num; i++ ) {
    if( strcmp( buf, kernel_name[i] )==0 ) return( i );
  }

  return(-1);
}

/* --------------------------------------------------------------------
 Computes kernel function for a-th and b-th.

 The base address for the 1st argument is dataA and for the 2nd
 argument is dataB.
-------------------------------------------------------------------- */
double kernel( long a, long b )
{
   double c = 0;
   double *c1=&c, *c2=&c;
   c2++;

   ker_cnt++;

   switch( ker ) {
      /* linear kernel */
      case 0:
         c = dot_prod( a, b );
         break;
      /* polynomial kernel */
      case 1:
         c = pow( (dot_prod( a, b) + arg1[1]), arg1[0] );
         break;
      /* radial basis functions kernel*/
      case 2:
         c = exp( -0.5*sub_dot_prod( a, b)/(arg1[0]*arg1[0]) );
         break;
      /* sigmoid */
      case 3:
         c = tanh( arg1[0]*dot_prod( a,b) + arg1[1] );
         break;
      /* bochner */
      case 4:
		  c = bochner(a,b);
         break;
      /* CosineMixture */
      case 5:
		  c = CosineMixture(a,b);
         break;

      default:
         c = 0;
   }
   return( c );
}

